@@ -34,14 +34,14 @@
 #include <unistd.h>
 #include <fcntl.h>
 #include "types.h"
+#include "util.h"
+#include "ttyio.h"
+#include "dynload.h"
+
 #ifdef IS_MODULE
-  #include "g10lib.h"
   #define _(a) (a)
 #else
-  #include "util.h"
-  #include "ttyio.h"
   #include "i18n.h"
-  #include "dynload.h"
 #endif
 
 static int open_device( const char *name, int minor );
@@ -71,12 +71,8 @@ open_device( const char *name, int minor )
 	g10_log_fatal("can't open %s: %s\n", name, strerror(errno) );
     if( fstat( fd, &sb ) )
 	g10_log_fatal("stat() off %s failed: %s\n", name, strerror(errno) );
-  #if defined(__sparc__) && defined(__linux__)
-    #warning something is wrong with UltraPenguin /dev/random
-  #else
     if( !S_ISCHR(sb.st_mode) )
 	g10_log_fatal("invalid random device!\n" );
-  #endif
     return fd;
 }
 
@@ -149,7 +145,7 @@ gather_random( void (*add)(const void*, size_t, int), int requester,
 #ifndef IS_MODULE
 static
 #endif
-const char * const gnupgext_version = "RNDLINUX ($Revision: 1.5 $)";
+const char * const gnupgext_version = "RNDLINUX ($Revision: 1.7 $)";
 
 static struct {
     int class;
