@@ -53,10 +53,10 @@
 #include <errno.h>
 
 #include "types.h"  /* for byte and u32 typedefs */
-#include "g10lib.h"
 #ifndef IS_MODULE
 #include "dynload.h"
 #endif
+#include "util.h"
 
 #ifndef EAGAIN
   #define EAGAIN  EWOULDBLOCK
@@ -477,13 +477,14 @@ slow_poll(FILE *dbgfp, int dbgall, size_t *nbytes )
 
 			/* Try and estimate how much entropy we're getting
 			 * from a data source */
-			if (dataSources[i].usefulness)
+			if (dataSources[i].usefulness) {
 			    if (dataSources[i].usefulness < 0)
 				total = (dataSources[i].length + 999)
 					/ -dataSources[i].usefulness;
 			    else
 				total = dataSources[i].length
 					/ dataSources[i].usefulness;
+			}
 			if( dbgfp )
 			    fprintf(dbgfp,
 			       "%s %s contributed %d bytes, "
@@ -756,7 +757,7 @@ gather_random( void (*add)(const void*, size_t, int), int requester,
 #ifndef IS_MODULE
 static
 #endif
-const char * const gnupgext_version = "RNDUNIX ($Revision: 1.11 $)";
+const char * const gnupgext_version = "RNDUNIX ($Revision: 1.13 $)";
 
 
 static struct {
