@@ -68,7 +68,7 @@ secret_key_list( int nnames, char **names )
 	STRLIST list = NULL;
 	for( ; nnames ; nnames--, names++ )
 	    add_to_strlist( &list, *names );
-	list_one( list, 0 );
+	list_one( list, 1 );
 	free_strlist( list );
     }
 }
@@ -132,7 +132,7 @@ list_one( STRLIST names, int secret )
 	}
 	do {
 	    merge_keys_and_selfsig( keyblock );
-	    list_keyblock( keyblock, 0 );
+	    list_keyblock( keyblock, 1 );
 	    release_kbnode( keyblock );
 	} while( !get_seckey_next( ctx, NULL, &keyblock ) );
 	get_seckey_end( ctx );
@@ -196,7 +196,7 @@ list_keyblock( KBNODE keyblock, int secret )
 	sk = NULL;
 	keyid_from_pk( pk, keyid );
 	if( opt.with_colons ) {
-	    trustletter = query_trust_info( pk );
+	    trustletter = query_trust_info( pk, NULL );
 	    printf("pub:%c:%u:%d:%08lX%08lX:%s:%s:",
 		    trustletter,
 		    nbits_from_pk( pk ),
@@ -222,8 +222,15 @@ list_keyblock( KBNODE keyblock, int secret )
     for( kbctx=NULL; (node=walk_kbnode( keyblock, &kbctx, 0)) ; ) {
 	if( node->pkt->pkttype == PKT_USER_ID ) {
 	    if( any ) {
-		if( opt.with_colons )
-		    printf("uid:::::::::");
+		if( opt.with_colons ) {
+		    byte namehash[20];
+
+		    rmd160_hash_buffer( namehash,
+					node->pkt->pkt.user_id->name,
+					node->pkt->pkt.user_id->len  );
+		    trustletter = query_trust_info( pk, namehash );
+		    printf("uid:%c::::::::", trustletter);
+		}
 		else
 		    printf("uid%*s", 28, "");
 	    }
