@@ -178,10 +178,13 @@ proc_pubkey_enc( CTX c, PACKET *pkt )
     c->last_was_session_key = 1;
     enc = pkt->pkt.pubkey_enc;
     /*printf("enc: encrypted by a pubkey with keyid %08lX\n", enc->keyid[1] );*/
+    /* Hmmm: why do I have this algo check here - anyway there is
+     * function to check it. */
     if( is_ELGAMAL(enc->pubkey_algo)
 	|| enc->pubkey_algo == PUBKEY_ALGO_DSA
 	|| is_RSA(enc->pubkey_algo)  ) {
-	if ( !c->dek ) {
+	if ( !c->dek && ((!enc->keyid[0] && !enc->keyid[1])
+			  || !seckey_available( enc->keyid )) ) {
 	    c->dek = m_alloc_secure( sizeof *c->dek );
 	    if( (result = get_session_key( enc, c->dek )) ) {
 		/* error: delete the DEK */
@@ -199,6 +202,10 @@ proc_pubkey_enc( CTX c, PACKET *pkt )
 	    log_info( _("public key encrypted data: good DEK\n") );
     }
     else {
+	/* fixme: defer this message until we have parsed all packets of
+	 * this type - do this by building a list of keys with their stati
+	 * and store it with the conetxt.  do_proc_packets can then use
+	 * this list to display some information */
 	log_error(_("public key decryption failed: %s\n"), g10_errstr(result));
     }
     free_packet(pkt);
@@ -474,7 +481,7 @@ list_node( CTX c, KBNODE node )
 	    keyid_from_pk( pk, keyid );
 	    if( mainkey ) {
 		c->local_id = pk->local_id;
-		c->trustletter = query_trust_info( pk );
+		c->trustletter = query_trust_info( pk, NULL );
 	    }
 	    printf("%s:%c:%u:%d:%08lX%08lX:%s:%s:",
 		    mainkey? "pub":"sub",
@@ -733,6 +740,7 @@ do_proc_packets( CTX c, IOBUF a )
 	if( opt.list_packets ) {
 	    switch( pkt->pkttype ) {
 	      case PKT_PUBKEY_ENC:  proc_pubkey_enc( c, pkt ); break;
+	      case PKT_SYMKEY_ENC:  proc_symkey_enc( c, pkt ); break;
 	      case PKT_ENCRYPTED:   proc_encrypted( c, pkt ); break;
 	      case PKT_COMPRESSED:  proc_compressed( c, pkt ); break;
 	      default: newpkt = 0; break;
