@@ -50,7 +50,7 @@ show_paths( ulong lid, int only_first )
     last_level = 0;
     while( (level=enum_cert_paths( &context, &lid, &otrust, &validity)) != -1){
 	char *p;
-	int rc;
+	int c, rc;
 	size_t n;
 	u32 keyid[2];
 	PKT_public_key *pk ;
@@ -77,7 +77,7 @@ show_paths( ulong lid, int only_first )
 		  level*2, "",
 		  nbits_from_pk( pk ), pubkey_letter( pk->pubkey_algo ),
 		  (ulong)keyid[1], lid, datestr_from_pk( pk ) );
-     #if 0
+
 	c = trust_letter(otrust);
 	if( c )
 	    putchar( c );
@@ -90,8 +90,6 @@ show_paths( ulong lid, int only_first )
 	else
 	    printf( "%02x", validity );
 	putchar(' ');
-      #endif
-
 
 	p = get_user_id( keyid, &n );
 	tty_print_string( p, n ),
@@ -241,6 +239,7 @@ _("Could not find a valid trust path to the key.  Let's see whether we\n"
 	     * those values from then on
 	     */
 	    otrust = get_ownertrust( lid );
+	    /* fixme: and the validity? */
 	}
 	if( otrust == TRUST_UNDEFINED ) {
 	    any_undefined=1;
@@ -296,12 +295,14 @@ do_we_trust( PKT_public_key *pk, int trustlevel )
 						      g10_errstr(rc) );
 	    return 0; /* no */
 	}
-	rc = check_trust( pk, &trustlevel );
+	rc = check_trust( pk, &trustlevel, NULL );
 	if( rc )
 	    log_fatal("trust check after insert failed: %s\n",
 						      g10_errstr(rc) );
-	if( trustlevel == TRUST_UNKNOWN || trustlevel == TRUST_EXPIRED )
-	    BUG();
+	if( trustlevel == TRUST_UNKNOWN || trustlevel == TRUST_EXPIRED ) {
+	    log_debug("do_we_trust: oops at %d\n", __LINE__ );
+	    return 0;
+	}
 	return do_we_trust( pk, trustlevel );
 
       case TRUST_EXPIRED:
@@ -318,7 +319,7 @@ do_we_trust( PKT_public_key *pk, int trustlevel )
 
 	    rc = add_ownertrust( pk, &quit );
 	    if( !rc && !quit ) {
-		rc = check_trust( pk, &trustlevel );
+		rc = check_trust( pk, &trustlevel, NULL );
 		if( rc )
 		    log_fatal("trust check after add_ownertrust failed: %s\n",
 							      g10_errstr(rc) );
@@ -367,11 +368,26 @@ do_we_trust( PKT_public_key *pk, int trustlevel )
 static int
 do_we_trust_pre( PKT_public_key *pk, int trustlevel )
 {
-    int rc = do_we_trust( pk, trustlevel );
+    int rc;
+
+    rc = do_we_trust( pk, trustlevel );
 
     if( (trustlevel & TRUST_FLAG_REVOKED) && !rc )
 	return 0;
     else if( !opt.batch && !rc ) {
+	char *p;
+	u32 keyid[2];
+	size_t n;
+
+	keyid_from_pk( pk, keyid);
+	tty_printf( "%4u%c/%08lX %s \"",
+		  nbits_from_pk( pk ), pubkey_letter( pk->pubkey_algo ),
+		  (ulong)keyid[1], datestr_from_pk( pk ) );
+	p = get_user_id( keyid, &n );
+	tty_print_string( p, n ),
+	m_free(p);
+	tty_printf("\"\n\n");
+
 	tty_printf(_(
 "It is NOT certain that the key belongs to its owner.\n"
 "If you *really* know what you are doing, you may answer\n"
@@ -380,6 +396,10 @@ do_we_trust_pre( PKT_public_key *pk, int trustlevel )
 	if( cpr_get_answer_is_yes("untrusted_key.override",
 				  _("Use this key anyway? "))  )
 	    rc = 1;
+
+	/* Hmmm: Should we set a flag to tell the user the user about
+	 *	 his decision the next time he encrypts for this recipient?
+	 */
     }
     else if( opt.always_trust && !rc ) {
 	log_info(_("WARNING: Using untrusted key!\n"));
@@ -410,7 +430,7 @@ check_signatures_trust( PKT_signature *sig )
     }
 
   retry:
-    rc = check_trust( pk, &trustlevel );
+    rc = check_trust( pk, &trustlevel, NULL );
     if( rc ) {
 	log_error("check trust failed: %s\n", g10_errstr(rc));
 	goto leave;
@@ -431,7 +451,7 @@ check_signatures_trust( PKT_signature *sig )
 						      g10_errstr(rc) );
 	    goto leave;
 	}
-	rc = check_trust( pk, &trustlevel );
+	rc = check_trust( pk, &trustlevel, NULL );
 	if( rc )
 	    log_fatal("trust check after insert failed: %s\n",
 						      g10_errstr(rc) );
@@ -571,7 +591,7 @@ build_pk_list( STRLIST remusr, PK_LIST *ret_pk_list, unsigned use )
 	    else if( !(rc=check_pubkey_algo2(pk->pubkey_algo, use)) ) {
 		int trustlevel;
 
-		rc = check_trust( pk, &trustlevel );
+		rc = check_trust( pk, &trustlevel, NULL );
 		if( rc ) {
 		    log_error("error checking pk of `%s': %s\n",
 						      answer, g10_errstr(rc) );
@@ -596,6 +616,7 @@ build_pk_list( STRLIST remusr, PK_LIST *ret_pk_list, unsigned use )
 	}
     }
     else {
+	any_recipients = 0;
 	for(; remusr; remusr = remusr->next ) {
 	    if( (remusr->flags & 1) )
 		continue; /* encrypt-to keys are already handled */
@@ -609,7 +630,7 @@ build_pk_list( STRLIST remusr, PK_LIST *ret_pk_list, unsigned use )
 	    else if( !(rc=check_pubkey_algo2(pk->pubkey_algo, use )) ) {
 		int trustlevel;
 
-		rc = check_trust( pk, &trustlevel );
+		rc = check_trust( pk, &trustlevel, NULL );
 		if( rc ) {
 		    free_public_key( pk ); pk = NULL;
 		    log_error(_("%s: error checking key: %s\n"),
@@ -650,6 +671,23 @@ build_pk_list( STRLIST remusr, PK_LIST *ret_pk_list, unsigned use )
 }
 
 
+
+static int
+algo_available( int preftype, int algo )
+{
+    if( preftype == PREFTYPE_SYM ) {
+	return algo && !check_cipher_algo( algo );
+    }
+    else if( preftype == PREFTYPE_HASH ) {
+	return algo && !check_digest_algo( algo );
+    }
+    else if( preftype == PREFTYPE_COMPR ) {
+	return !algo || algo == 1 || algo == 2;
+    }
+    else
+	return 0;
+}
+
 /****************
  * Return -1 if we could not find an algorithm.
  */
@@ -673,7 +711,7 @@ select_algo_from_prefs( PK_LIST pk_list, int preftype )
 
 	memset( mask, 0, 8 * sizeof *mask );
 	if( !pkr->pk->local_id ) { /* try to set the local id */
-	    query_trust_info( pkr->pk );
+	    query_trust_info( pkr->pk, NULL );
 	    if( !pkr->pk->local_id ) {
 		log_debug("select_algo_from_prefs: can't get LID\n");
 		continue;
@@ -726,9 +764,10 @@ select_algo_from_prefs( PK_LIST pk_list, int preftype )
 	    if( pref[j] == preftype ) {
 		any = 1;
 		if( (bits[pref[j+1]/32] & (1<<(pref[j+1]%32))) ) {
-		    /* fixme: check whether this algoritm is available */
-		    i = pref[j+1];
-		    break;
+		    if( algo_available( preftype, pref[j+1] ) ) {
+			i = pref[j+1];
+			break;
+		    }
 		}
 	    }
 	}
@@ -736,9 +775,10 @@ select_algo_from_prefs( PK_LIST pk_list, int preftype )
     if( !pref || !any ) {
 	for(j=0; j < 256; j++ )
 	    if( (bits[j/32] & (1<<(j%32))) ) {
-		/* fixme: check whether this algoritm is available */
-		i = j;
-		break;
+		if( algo_available( preftype, j ) ) {
+		    i = j;
+		    break;
+		}
 	    }
     }
   #if 0
