@@ -64,6 +64,46 @@ signature_check( PKT_signature *sig, MD_HANDLE digest )
 	rc = do_check( pk, sig, digest );
 
     free_public_key( pk );
+
+    if( !rc && is_status_enabled() ) {
+	/* This signature id works best with DLP algorithms because
+	 * they use a random parameter for every signature.  Instead of
+	 * this sig-id we could have also used the hash of the document
+	 * and the timestamp, but the drawback of this is, that it is
+	 * not possible to sign more than one identical document within
+	 * one second.	Some remote bacth processing applications might
+	 * like this feature here */
+	MD_HANDLE md;
+	u32 a = sig->timestamp;
+	int i, nsig = pubkey_get_nsig( sig->pubkey_algo );
+	byte *p, *buffer;
+
+	md = md_open( DIGEST_ALGO_RMD160, 0);
+	md_putc( digest, sig->pubkey_algo );
+	md_putc( digest, sig->digest_algo );
+	md_putc( digest, (a >> 24) & 0xff );
+	md_putc( digest, (a >> 16) & 0xff );
+	md_putc( digest, (a >>	8) & 0xff );
+	md_putc( digest,  a	   & 0xff );
+	for(i=0; i < nsig; i++ ) {
+	    unsigned n = mpi_get_nbits( sig->data[i]);
+
+	    md_putc( md, n>>8);
+	    md_putc( md, n );
+	    p = mpi_get_buffer( sig->data[i], &n, NULL );
+	    md_write( md, p, n );
+	    m_free(p);
+	}
+	md_final( md );
+	p = make_radix64_string( md_read( md, 0 ), 20 );
+	buffer = m_alloc( strlen(p) + 40 );
+	sprintf( buffer, "%s %s", p, strtimestamp( sig->timestamp ) );
+	write_status_text( STATUS_SIG_ID, buffer );
+	m_free(buffer);
+	m_free(p);
+	md_close(md);
+    }
+
     return rc;
 }
 
