@@ -4,21 +4,39 @@
 PGM=GnuPG
 DIE=no
 
+autoconf_vers=2.13
+automake_vers=1.4
+aclocal_vers=1.4
+
 if (autoconf --version) < /dev/null > /dev/null 2>&1 ; then
-    :
+    if (autoconf --version | awk 'NR==1 { if( $3 >= '$autoconf_vers') \
+			       exit 1; exit 0; }');
+    then
+       echo "**Error**: "\`autoconf\'" is too old."
+       echo '           (version ' $autoconf_vers ' or newer is required)'
+       DIE="yes"
+    fi
 else
     echo
     echo "**Error**: You must have "\`autoconf\'" installed to compile $PGM."
-    echo '           (version 2.10 or newer is required)'
+    echo '           (version ' $autoconf_vers ' or newer is required)'
     DIE="yes"
 fi
 
 if (automake --version) < /dev/null > /dev/null 2>&1 ; then
+  if (automake --version | awk 'NR==1 { if( $4 >= '$automake_vers') \
+			     exit 1; exit 0; }');
+     then
+     echo "**Error**: "\`automake\'" is too old."
+     echo '           (version ' $automake_vers ' or newer is required)'
+     DIE="yes"
+  fi
   if (aclocal --version) < /dev/null > /dev/null 2>&1; then
-    if (aclocal --version | awk 'NR==1 { if( $4 >= 1.3 ) exit 1; exit 0; }');
+    if (aclocal --version | awk 'NR==1 { if( $4 >= '$aclocal_vers' ) \
+						exit 1; exit 0; }' );
     then
       echo "**Error**: "\`aclocal\'" is too old."
-      echo '           (version 1.3 or newer is required)'
+      echo '           (version ' $aclocal_vers ' or newer is required)'
       DIE="yes"
     fi
   else
@@ -27,11 +45,10 @@ if (automake --version) < /dev/null > /dev/null 2>&1 ; then
     echo "           installed doesn't appear recent enough."
     DIE="yes"
   fi
-
 else
     echo
     echo "**Error**: You must have "\`automake\'" installed to compile $PGM."
-    echo '           (version 1.3 or newer is required)'
+    echo '           (version ' $automake_vers ' or newer is required)'
     DIE="yes"
 fi
 
@@ -41,7 +58,8 @@ if (gettext --version </dev/null 2>/dev/null | awk 'NR==1 { split($4,A,"\."); \
     then
     echo "**Error**: You must have "\`gettext\'" installed to compile $PGM."
     echo '           (version 0.10.35 or newer is required; get'
-    echo '            ftp://alpha.gnu.org/gnu/gettext-0.10.35.tar.gz)'
+    echo '            ftp://alpha.gnu.org/gnu/gettext-0.10.35.tar.gz'
+    echo '            or install the latest Debian package)'
     DIE="yes"
 fi
 
@@ -51,7 +69,7 @@ if test "$DIE" = "yes"; then
 fi
 
 echo "Running gettextize...  Ignore non-fatal messages."
-echo "no" | gettextize --force 
+echo "no" | gettextize --force
 
 
 echo "Running aclocal..."
