@@ -22,11 +22,12 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include "mpi.h"
-#define _g10lib_INTERNAL 1
-#include "g10lib.h"
+#include "util.h"
 
 
-const char *g10u_revision_string(int dummy) { return "$Revision: 1.4 $"; }
+/* FIXME: The modules should use functions from libgcrypt */
+
+const char *g10u_revision_string(int dummy) { return "$Revision: 1.5 $"; }
 
 
 void *g10_malloc( size_t n )	     { return m_alloc( n ); }
